﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

using Infragistics.Win;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinGrid;

namespace GridControl
{
    /// <summary>
    /// Represents a user control for managing the message headers for the Inbox,
    /// Sent Items, and Deleted Items.
    /// </summary>
    public partial class ucMessageHeaders : UserControl
    {
        private const string FROM_COLUMN_KEY = "SenderName";
        private const string SUBJECT_COLUMN_KEY = "MessageSubject";
        private const string SIZE_COLUMN_KEY = "Size";
        private const string RECEIVED_COLUMN_KEY = "Received";
        private const string IMPORTANCE_LEVEL_COLUMN_KEY = "Importance";
        private const string ATTACHMENT_COLUMN_KEY = "Attachment";
        private const string MAIL_COLUMN_KEY = "MailIcon";

        private MessageHeaderInboxCollection _list = new MessageHeaderInboxCollection();

        /// <summary>
        /// Provides the ability to initialize the instance of the class.
        /// </summary>
        public ucMessageHeaders()
        {
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);

            InitializeComponent();
            UiHelper.InitializeGridSettings(ultraGridMessages);
            PopulateCollection();
        }

        public void SetDataSource()
        {
            ultraGridMessages.DataSource = null;
            ultraGridMessages.Layouts.Clear();
            ultraGridMessages.DataSource = _list;
        }

        private void ultraGridMessages_InitializeLayout(object sender, Infragistics.Win.UltraWinGrid.InitializeLayoutEventArgs e)
        {
            e.Layout.Bands[0].Columns.Add(IMPORTANCE_LEVEL_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(MAIL_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(ATTACHMENT_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(RECEIVED_COLUMN_KEY);
            e.Layout.Bands[0].Columns.Add(SIZE_COLUMN_KEY);

            //ultraGridMessages.DisplayLayout.Bands[0].Columns[ID_COLUMN_KEY].Hidden = true;
            SetColumnPosition(e.Layout.Bands[0].Columns);
            SetColumnAttributes(e.Layout.Bands[0].Columns);
            HideColumns(e.Layout.Bands[0].Columns);
        }

        /// <summary>
        /// Sets the visible position of the columns that are shown.
        /// </summary>
        /// <param name="columns">Collection of column objects.</param>
        private void SetColumnPosition(ColumnsCollection columns)
        {
            columns[IMPORTANCE_LEVEL_COLUMN_KEY].Header.VisiblePosition = 0;
            columns[MAIL_COLUMN_KEY].Header.VisiblePosition = 1;
            columns[ATTACHMENT_COLUMN_KEY].Header.VisiblePosition = 2;
            columns[FROM_COLUMN_KEY].Header.VisiblePosition = 3;
            columns[SUBJECT_COLUMN_KEY].Header.VisiblePosition = 4;
            columns[RECEIVED_COLUMN_KEY].Header.VisiblePosition = 5;
            columns[SIZE_COLUMN_KEY].Header.VisiblePosition = 6;
        }

        /// <summary>
        /// Sets the column attributes of the columns that are shown.
        /// </summary>
        /// <param name="columns">Collection of column objects.</param>
        private void SetColumnAttributes(ColumnsCollection columns)
        {
            UiHelper.SetColumnAttributes(columns[IMPORTANCE_LEVEL_COLUMN_KEY], 20, SystemIcons.Asterisk.ToBitmap());
            UiHelper.SetColumnAttributes(columns[MAIL_COLUMN_KEY], 20, SystemIcons.Asterisk.ToBitmap());
            UiHelper.SetColumnAttributes(columns[ATTACHMENT_COLUMN_KEY], 20, SystemIcons.Asterisk.ToBitmap());

            UiHelper.SetColumnAttributes(columns[SIZE_COLUMN_KEY], 35, HAlign.Right, "Size");
            UiHelper.SetColumnAttributes(columns[SUBJECT_COLUMN_KEY], 200, HAlign.Default, "Subject");
            UiHelper.SetColumnAttributes(columns[FROM_COLUMN_KEY], 120, HAlign.Default, "From");
            UiHelper.SetColumnAttributes(columns[RECEIVED_COLUMN_KEY], 65, HAlign.Right, "Received");
        }

        private void HideColumns(ColumnsCollection columns)
        {
            columns["MessageId"].Hidden = true;
            columns["MessageImportanceLevel"].Hidden = true;
            columns["SenderId"].Hidden = true;
            columns["RespondOption"].Hidden = true;
            columns["MarkedUnread"].Hidden = true;
            columns["DateReceived"].Hidden = true;
            columns["AttachmentsSize"].Hidden = true;
        }

        /// <summary>
        /// Event that occurs when the row is double-clicked. 
        /// </summary>
        /// <param name="sender">Reference to the object that called the event handler.</param>
        /// <param name="e">Class instance containing data pertinent to the event.</param>
        private void ultraGridMessages_DoubleClickRow(object sender, DoubleClickRowEventArgs e)
        {

        }

        private void ultraGridMessages_InitializeRow(object sender, InitializeRowEventArgs e)
        {
            e.Row.Cells[IMPORTANCE_LEVEL_COLUMN_KEY].Value =
                GetMessageImportanceLevelImage((int)e.Row.Cells["MessageImportanceLevel"].Value);

            e.Row.Cells[MAIL_COLUMN_KEY].Value = GetMailImage((int)e.Row.Cells["RespondOption"].Value, (bool)e.Row.Cells["MarkedUnread"].Value);

            e.Row.Cells[ATTACHMENT_COLUMN_KEY].Value = GetAttachmentsImage((long)e.Row.Cells["AttachmentsSize"].Value > 0);

            if (e.Row.Cells["DateReceived"].Value != null)
            {
                e.Row.Cells[RECEIVED_COLUMN_KEY].Value = ((DateTime)e.Row.Cells["DateReceived"].Value).ToUniversalTime();
            }

            e.Row.Cells[SIZE_COLUMN_KEY].Value = FormatHelper.FormatBytes((long)e.Row.Cells["AttachmentsSize"].Value);
            e.Row.CellAppearance.FontData.Bold = (bool)e.Row.Cells["MarkedUnread"].Value ? DefaultableBoolean.True : DefaultableBoolean.False;
        }

        private Bitmap GetMailImage(int respondOption, bool isUnread)
        {
            Bitmap image = null;

            switch (respondOption)
            {
                case RespondOption.NONE:
                    image = SystemIcons.Asterisk.ToBitmap();
                    break;

                case RespondOption.REPLY:
                    break;

                case RespondOption.REPLY_ALL:
                    break;

                case RespondOption.FORWARD:
                    break;

                default:
                    // Let it default to None, but log it as a warning
                    image = SystemIcons.Asterisk.ToBitmap();
                    break;
            }

            return image;
        }

        /// <summary>
        /// Gets the image for the message importance level column.
        /// </summary>
        /// <param name="importanceLevel">The importance level of the message.</param>
        /// <returns>The image to display or null.</returns>
        private Bitmap GetMessageImportanceLevelImage(int importanceLevel)
        {
            Bitmap image = null;

            switch (importanceLevel)
            {
                case MessageImportanceLevel.LOW:
                    image = SystemIcons.Asterisk.ToBitmap();
                    break;

                case MessageImportanceLevel.NORMAL:
                    break;

                case MessageImportanceLevel.HIGH:
                    image = SystemIcons.Asterisk.ToBitmap();
                    break;

                default:
                    break;
            }

            return image;
        }

        /// <summary>
        /// Gets the image of the row if the message has attachments.
        /// </summary>
        /// <param name="hasAttachments">Flag indicating whether the message has attachments.</param>
        /// <returns>The image to display or null.</returns>
        private Bitmap GetAttachmentsImage(bool hasAttachments)
        {
            Bitmap image = null;

            if (hasAttachments)
            {
                image = SystemIcons.Asterisk.ToBitmap();
            }

            return image;
        }

        private void PopulateCollection()
        {
            var msg = new MessageHeaderInbox();

            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 1;
            msg.AttachmentsSize = 0;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = false;
            msg.MessageSubject = "message #1";
            msg.RespondOption = 0;
            msg.SenderName = "Goofy 1";
            _list.Add(msg);

            msg = new MessageHeaderInbox();
            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 2;
            msg.AttachmentsSize = 4096;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = true;
            msg.MessageSubject = "message #2";
            msg.RespondOption = 0;
            msg.SenderName = "Goofy 2";
            _list.Add(msg);

            msg = new MessageHeaderInbox();
            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 3;
            msg.AttachmentsSize = 0;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = true;
            msg.MessageSubject = "message #3";
            msg.RespondOption = 0;
            msg.SenderName = "Goofy 3";
            _list.Add(msg);

            msg = new MessageHeaderInbox();
            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 1;
            msg.AttachmentsSize = 4096;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = false;
            msg.MessageSubject = "message #4";
            msg.RespondOption = 0;
            msg.SenderName = "Goofy 4";
            _list.Add(msg);

            msg = new MessageHeaderInbox();
            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 2;
            msg.AttachmentsSize = 0;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = false;
            msg.MessageSubject = "message #5";
            msg.RespondOption = 0;
            msg.SenderName = "Goofy 5";
            _list.Add(msg);
        }

        private void ultraButton1_Click(object sender, EventArgs e)
        {
            var msg = new MessageHeaderInbox();

            msg.MessageId = Guid.NewGuid();
            msg.MessageImportanceLevel = 1;
            msg.AttachmentsSize = 0;
            msg.DateReceived = DateTime.UtcNow;
            msg.MarkedUnread = false;
            msg.MessageSubject = "new message";
            msg.RespondOption = 0;
            msg.SenderName = "Donald Duck";

            _list.Insert(0, msg);
            ultraGridMessages.Rows.Refresh(RefreshRow.ReloadData);
            //ultraGridMessages.DataBind();
        }
    }
}
